#pragma once

// C++ system level
#include <string>
#include <cinttypes>
#include <functional>

// ESP32 specific
// #include "freertos/FreeRTOS.h"
// #include "freertos/queue.h"

// project specific
// #include "LambdaTask.hpp"

// qthing stuff
#include <qthing>



namespace SiliconTorch {

  namespace FxCyanRGB8 {

    typedef struct __attribute__((__packed__)) {
      uint8_t r;
      uint8_t g;
      uint8_t b;
    } Pixel;

    extern const std::string HEADER;

    class FxCyanRGB8 : public qthing::Animation<qthing::RGB> {
      public:
        FxCyanRGB8(uint16_t leds, uint16_t startIdx = 0);

        void getAnimator();

        void requestBufferSwap();

        // Register qthing UDP packet handler
        // mainly used for testing purposes
        void registerUDPHandler();

        // Strip the header before calling this function!
        void handlePacket(uint8_t* buffer, std::size_t length);

        // qthing::Animator
        void step();
        qthing::RGB render(uint16_t addr);

      private:

        qthing::RGB* buffer0;
        qthing::RGB* buffer1;

        bool swapRequested = false;
        bool udpHandlerRegistered = false;

        uint16_t leds;
        uint16_t startIdx;


        void swapBuffers();  // TODO: visibility…?


    };
  }
}
