#!/usr/bin/env nix-shell
#!nix-shell -i python -p python3


import sys
import time
import struct
from socket import *


if len(sys.argv) < 2:
  print(f'Usage: {sys.argv[0]} HOST')
  exit(1)


port = 4213
host = sys.argv[1]
print(f'Using host[ {host} ]')


def sendRaw(data: bytes, _socket = socket(AF_INET, SOCK_DGRAM)):
  try:
    _socket.sendto(data, (host, port))
  except:
    pass


class RGB:
  def __init__(self, r: int = 0, g: int = 0, b: int = 0):
    self.r = r
    self.g = g
    self.b = b

  def toBytes(self):
    return struct.pack('BBB', self.r, self.g, self.b)



def sendPacket(startIdx: int = 0, pixel: list = [], header: bytes = b'fxCyanRGB8'):
  data = header + struct.pack('>H', startIdx)

  for px in pixel:
    data += px.toBytes()

  print(f'Sending |pixel|[ {len(pixel)} ]…')
  sendRaw(data=data)


if __name__ == '__main__':
  white = RGB(255, 255, 255)
  black = RGB(  0,   0,   0)
  red   = RGB(255,   0,   0)
  green = RGB(  0, 255,   0)
  blue  = RGB(  0,   0, 255)

  pixels = [
    white,
    red, green, blue,
    white
  ]

  FullHour = [
    red,
    green,
    blue
  ]

  fh = 0
  while True:
    sendPacket(pixel=pixels + [FullHour[fh]])

    fh += 1
    if fh >= len(FullHour): fh = 0

    time.sleep(0.25)
